/******************************************************************************
*******************************************************************************
**                                                                           **
**      C Swing - A graphical file and directory maintenance utility         **
**                                                                           **
*******************************************************************************
**                                                                           **
**      Original C coding by: S. J. Brown                                    **
**                                                                           **
**      Modifications by many others; see the documentation for a list       **
**                                                                           **
**      Copyright 1994, Harry Flowers and Foteos Macrides                    **
**                                                                           **
**      Terms of Copyright:  This software may be freely distributed         **
**          and used by anyone, providing the distributor provides it        **
**          free of charge or at a price which only covers distribution      **
**          costs.  It may not be sold.  Note that versions prior to         **
**          V3.7.4 were public domain, and are not subject to these terms.   **
**                                                                           **
**      Disclaimer: No warranties as to the use or reliability of this       **
**          software are given by the authors or distributors.  Use at       **
**          your own risk.                                                   **
**                                                                           **
*******************************************************************************
******************************************************************************/
 
/*
**  swing_15
*/
#include <stdio.h>
#include <smgdef.h>
#include <smgmsg.h>
#include <ssdef.h>
#include "more_descrip.h"
#include "swing.h"
#include "swing_refs.h"

display_line(ptr,line,scroll,direction)
LINE *ptr;
int line,scroll,direction;
{
    /*
    **  ptr - pointer to LINE structure containing info to display
    **  line - line no to display data on
    **  scroll - TRUE ==> scroll
    **  direction - if scroll this is the direction to scroll in
    **
    **  when we start, cursor is on current line
    */
    int i,rend,status,column,end_column,flag;

    /*
    **  scroll or clear
    */
    if (scroll)
    {
        status = smg$scroll_display_area(&display1,
                                         &1,&1,
                                         0,0,&direction);
        if (status != SS$_NORMAL)
            panic(status,"scroll display area","display line");
    }

    smg$begin_display_update(&display1);

    /*
    **  erase current line, cos we'll now overwrite it
    */
    status = smg$erase_line(&display1,&line,&1);
    if (status != SS$_NORMAL)
        panic(status,"erase line","display line");

    if (!ptr->dp)
    {
        /*
        **  this is a separator line between structures
        */
        column = 1;
        end_column = (2 * NAME_SHORT_SIZE + 1);

        status = smg$draw_line(&display1,
                               &line,
                               &column,
                               &line,
                               &end_column);
        if (status != SS$_NORMAL)
            panic(i,"drawing a line","display line");
    }
    else for (i = 0; i < MAX_LEVEL; i++)
    {
        column =(i*(NAME_SHORT_SIZE+1)) + 1;
        if (ptr->n_ptr[i])
        {
            /*
            **  if there is a node at the same level on next line down
            **  and she's not my sister underscore me , otherwise show me 
            **  as a normal node
            */
            if (ptr->forw_ptr && ptr->forw_ptr->n_ptr[i] &&
                !ptr->n_ptr[i]->sister)
            {
                rend = UNDERLINE_RENDITION;
            }
            else
            {
		rend = NORMAL_RENDITION;
            }

            status = smg$put_chars(&display1,
                                   DSC1(ptr->n_ptr[i]->short_name),
                                   &line,
                                   &column,
                                   0,
                                   0,
                                   0,
                                   &SMG$C_ASCII);
            if (status != SS$_NORMAL)
                panic(status,"put chars node","display line");

            status = smg$change_rendition(&display1,
                                          &line,
                                          &column,
                                          &1,
                                          &NAME_SHORT_SIZE,
                                          &rend);
            if (status != SS$_NORMAL)
                panic(status,"change rendition","display line");
        }

        if (ptr->connections[i])
        {       
            column =((i+1)*(NAME_SHORT_SIZE+1));

            /*
	    **  work out the flag for the call - the elements of
	    **  structure ptr->connections are defined in swing.h
	    */
            if (ptr->connections[i])
	    {
		flag = 0;

		if (ptr->connections[i] & C_UP)
		    flag |= SMG$M_UP;
		if (ptr->connections[i] & C_DOWN)
		    flag |= SMG$M_DOWN;
		if (ptr->connections[i] & C_LEFT)
		    flag |= SMG$M_LEFT;
		if (ptr->connections[i] & C_RIGHT)
		    flag |= SMG$M_RIGHT;

		if (ptr->connections[i] == C_DIAMOND)
		    flag = 0;

                status = smg$draw_char(&display1,
                                       &flag,
                                       &line,
                                       &column,
                                       0,
                                       0);

                if (status != SS$_NORMAL)
                    panic(status,"draw char","display line");
	    }
        }
    }
    smg$end_display_update(&display1);
}
